#include "ColorPanel.h"
#include "ColorStateView.h"
#include "MessageConstants.h"

static const BPoint SIZE(260,100);
static const BRect FRAME(0,0,259,99);
static const BRect FRAME_SYNC(10,82,110,99);
enum{
	M_COLOR='Colr',
	M_SELECT='Slct',
	M_AUTO = 'Auto'
};

ColorPanel::ColorPanel(BPoint p, BMessenger target)
	: BView
		( BRect(p, p+SIZE)
		, "Color"
		, B_FOLLOW_LEFT|B_FOLLOW_TOP
		, 0//B_WILL_DRAW
		)
	, v_color_state(NULL)
	, v_color_control(NULL)
	, v_sync(NULL)
	, target(target)
{
	SetViewColor(0xC0,0xC0,0xC0);
	v_color_state = new ColorStateView(BPoint(0,0));
	v_color_control = new BColorControl(
		BPoint(0,16), B_CELLS_32x8, 8, "color control", new BMessage(M_COLOR));
	v_sync = new SimpleCheckBox(
		FRAME_SYNC, "Synchronize", new BMessage(M_AUTO));
	AddChild(v_color_state);
	AddChild(v_color_control);
	AddChild(v_sync);
}

void
ColorPanel::GetPreferredSize(float*x,float*y)
{
	*x=SIZE.x;
	*y=SIZE.y;
}

void
ColorPanel::AttachedToWindow()
{
	BView::AttachedToWindow();
	v_color_control->SetTarget(this);
	v_color_state->SetTarget(this);
	v_sync->SetTarget(this);
//	BMessage m(MC_GET_VALUES);
//	m.AddInt32("length", 16);
//	target.SendMessage(&m, this);
	
	BMessage m(MC_SET_VALUES);
	m.AddData("colors", B_RGB_COLOR_TYPE, v_color_state->Colors(), 
				sizeof(rgb_color)*v_color_state->NUM_COLORS);
	target.SendMessage(&m);
}
void
ColorPanel::MessageReceived(BMessage* msg)
{
	switch(msg->what)
	{
//	case MC_GET_VALUES:
//		{
//			rgb_color* data;
//			ssize_t size=0;
//			msg->FindData("colors",B_RGB_COLOR_TYPE, &data, &size);
//			size = size/sizeof(rgb_color);
//			size = min_c(16, size);
//			for(int i=0;i<size;i++)
//				v_color_state->SetColor(data[i], i);
//
//		}
//		break;

	case M_AUTO:
		if(v_sync->Value()==B_CONTROL_OFF) 
			v_color_state->SetMessage(NULL);
		else
			v_color_state->SetMessage(new BMessage(M_SELECT));
		break;
	case M_COLOR:
		{
			rgb_color c = v_color_control->ValueAsColor();
			v_color_state->SetColor(c);
			int32 i = c.red;i<<=8;i|=c.green;i<<=8;i|=c.blue;i<<=8;
			i|=v_color_state->Selection();
			BMessage msg(MC_SET_VALUE);
			msg.AddInt32("color", i);
			target.SendMessage(&msg);
		}
		break;
	case M_SELECT:
		v_color_control->SetValue(v_color_state->Color());
		break;
	default:
			BView::MessageReceived(msg);
	}
}

