
#include "ColorStateView.h"


static const BRect CELL_RECT(1,1,15,15);
static const BRect FRAME_COLOR_STATE(0,0,255,15);
static int		Pt2Id(BPoint p){return p.x/16;}
//static BPoint	Id2Pt(int id){return BPoint((id%16)*16, 0);}
static BRect	Id2Rect(int id){return CELL_RECT.OffsetByCopy(id*16,0);}
static int	Normalize(int i)
{
	int n=ColorStateView::NUM_COLORS;
	return (i>=0)?(i%n):(n-(-i)%n);
}

ColorStateView::ColorStateView(BPoint loc)
	: BView
		( FRAME_COLOR_STATE.OffsetByCopy(loc.x, loc.y) 
		, "color state view"
		, B_FOLLOW_LEFT|B_FOLLOW_TOP
		, B_WILL_DRAW)
	, selection(0)
{
	SetViewColor(0xC0,0xC0,0xC0);
	for(int i=0;i<16;i++)
	{
		colors[i].red=i*16;
		colors[i].green=255;
		colors[i].blue=0;
		colors[i].alpha=0xff;
	}
		colors[0].green=0;
}


void
ColorStateView::GetPreferredSize(float*w, float*h)
{
	*w = FRAME_COLOR_STATE.Width(); *h= FRAME_COLOR_STATE.Height();
}

void
ColorStateView::Draw(BRect rect)
{
	SetDrawingMode(B_OP_COPY);
	for(int id=0;id<NUM_COLORS;id++)
	{
		SetHighColor(colors[id]);
		FillRect(Id2Rect(id)&rect);
	}
	SetDrawingMode(B_OP_INVERT);
	StrokeRect(Id2Rect(selection).InsetBySelf(1,1));
}
void
ColorStateView::MouseDown(BPoint p)
{
	Invalidate(Id2Rect(selection));
	selection = Normalize(Pt2Id(p));
	Invalidate(Id2Rect(selection));
	Invoke();
}
void
ColorStateView::SetColor(rgb_color color)
{
	SetColor(color, selection);
}
void
ColorStateView::SetColor(rgb_color color, int id)
{
	id = Normalize(id);
	colors[id]=color;
	Invalidate(Id2Rect(id));
}


rgb_color
ColorStateView::Color()
{
	return Color(selection);
}

rgb_color
ColorStateView::Color(int id)
{
	return colors[Normalize(id)];
}


