#include "RenderPanel.h"
#include "MessageConstants.h"
#include "SpeedSlider.h"
#include "ColorPanel.h"


static const BRect FRAME_SCALE(10,10,110,29);
static const BRect FRAME_PERIOD(10,40,110,59);
static const BRect FRAME_METHOD(120,10,260,29);

static const BRect FRAME_FLASH(115,40,260,59);

static const BRect FRAME_SPEED(10, 70, 260, 129);

static const BRect FRAME_COLOR(10, 140, 260, 284);

static const BPoint WHOLE_SIZE(262, 262);
static const BRect RECT_VIEW(0, 0, 262, 140);

	static BMenuItem* create_num_item(uint32 command, const char* key, int32 n);
	static void invoke_menu(BMenu*);


RenderPanel::RenderPanel(BPoint p, const BMessenger& target)
	: BView(RECT_VIEW.OffsetByCopy(p)
		, "Rendering"
		, B_FOLLOW_ALL
		, 0//B_WILL_DRAW
		)
	, m_period(NULL)
	, m_scale(NULL)
	, m_method(NULL)
	, v_speed(NULL)
{
	SetViewColor(0xC0,0xC0,0xC0);
	{
		BMenu* m = new BPopUpMenu("Scale");
		static const uint8 scales[]={1,2,4,8};
		for(int i=0;i<4;i++)
			m->AddItem(create_num_item(MR_SET_SCALE, "scale", scales[i]));
		m->ItemAt(0)->SetMarked(true);
		m->SetTargetForItems(target);
		m_scale=m;
	}
	{
		BMenu* m = new BPopUpMenu("Period");
		for(int i=1;i<21;i++)
			m->AddItem(create_num_item(MP_SET_PERIOD, "period", i));
		m->ItemAt(0)->SetMarked(true);
		m->SetTargetForItems(target);
		m_period = m;
	}
	{
		BMenu* m = new BPopUpMenu("Method");
			m->AddItem(new BMenuItem("Gradation",new BMessage(MR_SET_METHOD)));
			m->AddItem(new BMenuItem("Generation",new BMessage(MR_SET_METHOD)));
			m->AddItem(new BMenuItem("Rotation",new BMessage(MR_SET_METHOD)));
			m->AddItem(new BMenuItem("Monochrome",new BMessage(MR_SET_METHOD)));
		m->ItemAt(0)->SetMarked(true);
		m->SetTargetForItems(target);
		m_method=m;
	}
	BMenuField* mf;
	AddChild(mf = new BMenuField(FRAME_SCALE, "scale mf", "Dot Size:", m_scale));
	mf->SetDivider(60);
	AddChild(mf = new BMenuField(FRAME_PERIOD, "period mf", "Period:", m_period));
	mf->SetDivider(60);
	AddChild(mf = new BMenuField(FRAME_METHOD, "method mf", "Method:", m_method));
	mf->SetDivider(50);

	BCheckBox* v_flash = new SimpleCheckBox(
		FRAME_FLASH, "Avoid screen flashing", new BMessage(MP_SET_AVOID_FLASH));
	v_flash->SetValue(B_CONTROL_ON);
	v_flash->SetTarget(target);
	AddChild(v_flash);
	
	v_speed = new SpeedSlider(FRAME_SPEED);
	v_speed->SetTarget(target);
	AddChild(v_speed);

	BView* color_panel = new ColorPanel(BPoint(0,140), target);
	AddChild(color_panel);

	invoke_menu(m_scale);
}

void
RenderPanel::GetPreferredSize(float* x, float* y)
{
	*x=WHOLE_SIZE.x;
	*y=WHOLE_SIZE.y;

}

	static BMenuItem* create_num_item(uint32 command, const char* key, int32 n)
	{
		BString s("");
		s<<n;
		BMessage* msg = new BMessage(command);
		msg->AddInt32(key, n);
		return new BMenuItem(s.String(), msg);
	}


	static void invoke_menu(BMenu* m)
	{
		BMenuItem* item = m->FindMarked();
		if(item==NULL) return;
		BMessenger msgr = item->Messenger();
		if(!msgr.IsValid()) return;
		BMessage* msg = item->Message();
		if(msg==NULL) return;
		BMessage message(msg);
		msgr.SendMessage(&message);
	}
