#ifndef RULE_LIST_VIEW_H
#define RULE_LIST_VIEW_H

#include "Prefix.h"

class RuleItem:public BStringItem
{
	uint32 rule;
public:
	RuleItem(const char* name, uint32 rule)
		: BStringItem(name, 1)
		, rule(rule){}

	void SetName(const char* n) {SetText(n);}
	const char* Name() const {return Text();}
	void SetRule(uint32 r) {rule = r;}
	uint32	Rule() const{return rule;}
	RuleItem& operator=(const RuleItem& item)
	{
		SetName(item.Name());
		SetRule(item.Rule());
		return *this;
	}
};


enum{
	M_DIALOG_ADD = 'DgAd',
	M_DIALOG_RENAME = 'DgRn',

	M_COPY = 'RlR0',
	M_CUT,
	M_PASTE,
	M_REMOVE,
	M_RESTORE,
	M_RENAME,
	M_ADD,
};



class RuleListView :public BListView
{
	static RuleItem	clip_board;
	static BLocker	clip_board_lock;

	bool is_dirty;

public:
	static const char* DATA_HEADER;//"%LifeHex vers.1.0";
	static const int MAX_ITEMS=1000;

	RuleListView(BRect frame);
	~RuleListView();
	
	
	bool IsDirty() const;
	void SetDirty(bool);
	bool HasSelection() const;
	const char* RuleName(int index = -1) const;
	uint32 RuleData(int index = -1) const;
	void SetRuleName(const char*, int index = -1);
	void SetRuleData(uint32 rule, int index = -1);
	void RemoveAll();
	void RemoveRule();
	void RemoveRule(int index);
	void SetRule(const char* name, uint32 rule, int index = -1);
	void AddRule(const char* name, uint32 rule);
	void AddRule(const char* name, uint32 rule, int index);
	
	void CopyRule();
	void PasteRule();

	void DialogRename();
	void DialogAdd(BMessage*);
	void AddRule(BMessage*);
	void Rename(BMessage*);

	void SelectionChanged();
	status_t Invoke(BMessage* msg);
	void MessageReceived(BMessage *msg);
	void KeyDown(const char*, int32);
	bool InitiateDrag(BPoint p, int32 index, bool selected);
	
	status_t LoadRules(BDataIO* in);
	status_t SaveRules(BDataIO* out);
};


#endif

