#include "RuleListWindow.h"

#include "RuleListView.h"
#include "LifeApplication.h"

//int32 RuleListWindow::num_windows(0);
BList RuleListWindow::window_list;
BLocker RuleListWindow::window_list_lock("list fo RuleListWindow");


enum{
	M_SAVE = 'RwSv',
	M_REVERT = 'RwRt',
	M_CLOSE = B_QUIT_REQUESTED,

	M_DIALOG_SAVE = 'DgSv',
	M_DIALOG_REVERT = 'Dgrt',

};

static const BRect FRAME_MBAR(0,0,100,19);
static const BRect FRAME_LIST(0,20,80,220);

RuleListWindow::RuleListWindow
	(BRect frame, const BMessenger&target)
	: BWindow(
		frame,
		"Untitled",
		B_DOCUMENT_WINDOW, 
		B_ASYNCHRONOUS_CONTROLS)
	, file_ref()
	, type(NEW)
	, save_panel(NULL)
{
	{
		BAutolock l(window_list_lock);
		if(l.IsLocked())
			window_list.AddItem(this);
	}
	SetSizeLimits(70,200,20,10000);

	float w = frame.Width(), h = frame.Height();
	v_bar = new BMenuBar(BRect(0,0,w,19), "rule list mbar");
		BMenu* m;
		m=new BMenu("File");
			m->AddItem(new BMenuItem("New", new BMessage(MA_NEW), 'N'));
			m->AddItem(new BMenuItem("Open", new BMessage(MA_OPEN), 'O'));
			m->AddSeparatorItem();
			m->AddItem(new BMenuItem("Quit", new BMessage(MA_QUIT), 'Q'));
			m->AddSeparatorItem();
			m->AddItem(new BMenuItem("About LifeHex...", new BMessage(MA_ABOUT)));
		m->SetTargetForItems(be_app);
			m->AddItem(new BMenuItem("Save", new BMessage(M_SAVE), 'S'),2);
			m->AddItem(new BMenuItem("Save As...", new BMessage(M_DIALOG_SAVE)),3);
			m->AddItem(new BMenuItem("Revert", new BMessage(M_DIALOG_REVERT)),4);
			m->AddItem(new BMenuItem("Close", new BMessage(M_CLOSE), 'W'),5);
		m_file = m;
		v_bar->AddItem(m);
		m=new BMenu("Edit");
			m->AddItem(new BMenuItem("Add Current Rule...", new BMessage(M_DIALOG_ADD), 'D'));
			m->AddItem(new BMenuItem("Cut", new BMessage(M_CUT), 'X'));
			m->AddItem(new BMenuItem("Copy", new BMessage(M_COPY), 'C'));
			m->AddItem(new BMenuItem("Paste", new BMessage(M_PASTE), 'V'));
			m->AddItem(new BMenuItem("Rename...", new BMessage(M_DIALOG_RENAME)));
			//m->AddItem(new BMenuItem("Restore", new BMessage(M_RESTORE)));
		v_bar->AddItem(m);
		m_edit=m;
	AddChild(v_bar);
	
	v_list = new RuleListView(BRect(0, 20, w-B_V_SCROLL_BAR_WIDTH, h));
		v_list->SetMessage(new BMessage(M_DIALOG_RENAME));
		v_list->SetTarget(target);
//		v_list->SetSelectionMessage(new BMessage(M_SELECT));
	BScrollView* s = new BScrollView
		( "rule list scroller"
		, v_list
		, B_FOLLOW_ALL_SIDES
		, 0
		, false
		, true
		);
	AddChild(s);
		m->SetTargetForItems(v_list);
}

RuleListWindow::~RuleListWindow()
{
	if(save_panel) delete save_panel;
	{
		BAutolock l(window_list_lock);
		if(l.IsLocked())
			window_list.RemoveItem(this);
	}
}

void
RuleListWindow::MessageReceived(BMessage *msg)
{
	switch(msg->what)
	{
	case M_DIALOG_REVERT:
		DialogRevert();
		break;
	case M_REVERT:
		Load(&file_ref);
		break;
	case M_SAVE:
		if(!v_list->IsDirty()||(type==READ_ONLY)) break;
		if(type==NEW)  DialogSave();
		else Save(&file_ref);	
		break;
	case M_DIALOG_SAVE:
		DialogSave();
		break;
	case B_CANCEL:
		if(save_panel) delete save_panel;
		save_panel=NULL;
		break;
	case B_REFS_RECEIVED://
		LoadRequested(msg);
		break;
	case B_SAVE_REQUESTED:
		SaveRequested(msg);
		break;
	default:
		BWindow::MessageReceived(msg);
	}
}





void
RuleListWindow::DialogSave()
{
	if(save_panel==NULL)
	{
		save_panel = new BFilePanel(B_SAVE_PANEL, new BMessenger(this));
		if(type!=NEW)
		{
			BEntry e(&file_ref);
			BDirectory d;
			e.GetParent(&d);
			save_panel->SetPanelDirectory(&d);
			save_panel->SetSaveText(Title());
		}
	}
	save_panel->Show();
}
void
RuleListWindow::DialogRevert()
{
	if(!v_list->IsDirty() || (type==NEW)) return;
	BString s("Revert to the last saved virsion of: \n   \"");
	s<<Title()<<"\"?";
	BAlert* alert = new BAlert
		( "Revert Alert"
		, s.String()
		, "Don't revert"
		, "Revert"
	);
	alert->SetShortcut(0, B_ESCAPE);
	int button = alert->Go();
	if(button==1)
		PostMessage(M_REVERT);
}


bool
RuleListWindow::Load(entry_ref* ref)
{
	BFile f(ref, B_READ_ONLY);
	if (
		(f.InitCheck() < B_OK) ||
		(v_list->LoadRules(&f)<B_OK)
	)
	{
		(new BAlert("Error", "Failed to load", "OK"))->Go();
		return false;
	}
	else
	{
		v_list->SetDirty(false);
		return true;
	}
}

bool
RuleListWindow::Save(entry_ref* ref)
{
	BFile f(ref, B_WRITE_ONLY|B_CREATE_FILE|B_ERASE_FILE);
	if (
		(f.InitCheck() < B_OK) ||
		(v_list->SaveRules(&f)<B_OK)
	)
	{
		(new BAlert("Error", "Failed to save", "OK"))->Go();
		return false;
	}
	else
	{
		BNodeInfo node(&f);
		node.SetType("text/plain");
		//node.SetPreferredApp("application/x-vnd.LifeHex");
		v_list->SetDirty(false);
		return true;
	}
}


void
RuleListWindow::SaveRequested(BMessage* msg)
{
	entry_ref ref;
	char* name;
	if(msg->FindString("name", &name)<B_OK) return;
	if(msg->FindRef("directory", &ref)<B_OK) return;

	BDirectory d(&ref);
	BEntry e(&d, name, false);
	e.GetRef(&ref);

	if(Save(&ref))
	{
		SetTitle(name);
		file_ref = ref;
		type=NORMAL;
	}
}


void
RuleListWindow::LoadRequested(BMessage* msg)
{
	if(msg->FindRef("refs", &file_ref)!=B_OK) return;
	{
		BEntry e(&file_ref, true);
		char name[B_FILE_NAME_LENGTH];
		e.GetName(name);
		SetTitle(name);
		e.GetRef(&file_ref);
	}
	if(Load(&file_ref)) type=NORMAL;
}



void
RuleListWindow::MenusBeginning()
{
	bool  t = v_list->IsDirty();
	m_file->ItemAt(2)->SetEnabled(t);//Save
	m_file->ItemAt(4)->SetEnabled(t);//Revert
}


bool
RuleListWindow::QuitRequested()
{
	if(!v_list->IsDirty()) return true;
	
	BString s("Save changes to the document: \n    \"");
	s<<Title()<<"\"?";
	BAlert* alert = new BAlert
		( "Close Alert"
		, s.String()
		, "Cancel"
		, "Don't save"
		, "Save"
		, B_WIDTH_AS_USUAL
		, B_OFFSET_SPACING
	);
	alert->SetShortcut(0, B_ESCAPE);
	int button = alert->Go();

	if(button==1) return true;
	if(button==2)
	{
		if((type==NORMAL)&&Save(&file_ref)) return true;
		PostMessage(M_DIALOG_SAVE);
	}
	return false;
}


//	Maintains the window list such that: 
//	if 2 windows x, y are shown as x upon y in the screen,
//	x comes after y in the list.
void
RuleListWindow::WindowActivated(bool active)
{
	if(active)
	{
		BAutolock l(window_list_lock);
		window_list.RemoveItem(this);
		window_list.AddItem(this);
	}
}

