#include "HexRule.h"
#include "RulePanel.h"
#include "MessageConstants.h"
#include "RuleStateView.h"

enum{
	M_ALTERNATE = 'Altr',
	M_CLEAR = 'Cler',
	M_FILL = 'Fill',
	M_ADD_RULE = 'Arle',
	M_PREV = 'Prev',
	M_NEXT = 'Next',
	M_RULE = 'Rule',
};

//static const BPoint SIZE_RULE_PANEL(415, 63);
//static const BRect FRAME_ALTER(10,70,80,95);
//static const BRect FRAME_CLEAR(90,70,160,95);
//static const BRect FRAME_FILL(170,70,240,95);
//static const BRect FRAME_RANDOM(250,70,320,95);

static const BPoint	MAX_SIZE(512,64);
static const BRect FRAME_BUTTONS(416,0,511,63);

static const BPoint SIZE_RULE_PANEL(511, 63);
static const BRect FRAME_ALTER(8,0,87,31);
static const BRect FRAME_RANDOM(8,32,87,63);

RulePanel::RulePanel(BPoint p, const BMessenger& target)
	: BView
		( BRect(p, SIZE_RULE_PANEL+p)
		, "Rule"
		, B_FOLLOW_LEFT|B_FOLLOW_TOP
		, 0//B_WILL_DRAW
		)
{
	SetViewColor(B_TRANSPARENT_COLOR);

	v_rule_state = new RuleStateView(BPoint(0,0), new BMessage(MG_SET_RULE));
	v_rule_state->SetValue(DEFAULT_RULE);
	v_rule_state->SetTarget(target);
	
	BView* v= new BView(FRAME_BUTTONS, "buttons",0,0);
	v_alter = new BButton(FRAME_ALTER,"alter","Alternate",new BMessage(M_ALTERNATE));
	v_random  = new BButton(FRAME_RANDOM,"randomize","Randomize",new BMessage(MG_RANDOMIZE));
	v->AddChild(v_alter);
	v->AddChild(v_random);
	v->SetViewColor(0xC0,0xC0,0xC0);
	v_random->SetTarget(target);
	
	AddChild(v);
	AddChild(v_rule_state);
}

RulePanel::~RulePanel()
{
}

void
RulePanel::AttachedToWindow()
{
	BView::AttachedToWindow();
	v_alter->SetTarget(this);
	v_rule_state->Messenger().SendMessage(MG_GET_RULE, this);
}

void
RulePanel::GetPreferredSize(float* w, float*h)
{
	*w = MAX_SIZE.x;
	*h = MAX_SIZE.y;
}

void
RulePanel::MessageReceived(BMessage* msg)
{
	switch(msg->what)
	{
	case MG_SET_RULE:
		{
			int32 rule;
			if(msg->FindInt32("rule", &rule)==B_OK)
			{
				v_rule_state->SetValue(rule);
				v_rule_state->Invoke();
			}
		}
		break;
	case MG_GET_RULE:
		{
			int32 rule;
			if(msg->FindInt32("rule", &rule)==B_OK)
				v_rule_state->SetValue((uint32)rule);
			else
			{
				BMessage m(MG_GET_RULE);
				m.AddInt32("rule", v_rule_state->Value());
				msg->SendReply(&m);
			}
		}
		break;
	case M_ALTERNATE:
		SetRule(AlternateRule(v_rule_state->Value()));
		break;
	case M_CLEAR:
		SetRule(0);
		break;
	case M_FILL:
		SetRule(-1);
		break;
	default:
		BView::MessageReceived(msg);
		break;
	}
}

void
RulePanel::SetRule(uint32 rule)
{
	v_rule_state->SetValue(rule);
	v_rule_state->Invoke();
}

uint32
RulePanel::Rule()
{
	return v_rule_state->Value();
}













//	BButton*		v_prev;
//	BButton*		v_next;
//	static const int	NUM_HISTORY=128;
//	static const uint32	EMPTY_RULE=0xffffffff;
//	static const uint32	RULE_DEFAULT=128;
//	int32	history[NUM_HISTORY];
//	unsigned	hist_current;
//		case M_NEXT:
//		{
//			hist_current = (hist_current+1)%NUM_HISTORY;
//			int32 rule = history[hist_current];
//			v_rule_state->SetValue(rule);
//			unsigned h = (hist_current+1)%NUM_HISTORY;
//			v_next->SetEnabled(history[h]!=EMPTY_RULE);
//			v_prev->SetEnabled(true);
//		}
//		break;

	
/*
		case M_SET_MESSENGER:
		{
			BMessenger messenger;
			if(msg->FindMessenger(v_rule_state->Name(), &messenger)==B_OK)
				v_rule_state->SetTarget(messenger);
			else BView::MessageReceived(msg);
		}
		break;


*/