#include "RuleStateView.h"

static const BRect cell_frame(0,0,31,31);
static int		Pt2Id(BPoint p);
static BPoint	Id2Pt(int id);

static const BRect FRAME(0,0,415,63);

RuleStateView::RuleStateView(BPoint loc, BMessage* msg)
	: BControl(
		BRect(FRAME.OffsetByCopy(loc)) 
		, "rule state view"
		, "Rule"
		, msg
		, B_FOLLOW_LEFT|B_FOLLOW_TOP
		, B_WILL_DRAW)
	, button_image(NULL)
	, last_changed(-1)
{
	SetViewColor(B_TRANSPARENT_COLOR);
	button_image = BTranslationUtils::GetBitmapFile("rule_control.tga");
}

RuleStateView::~RuleStateView()
{
	if(button_image!=NULL)delete button_image;
}

void
RuleStateView::GetPreferredSize(float* w, float* h)
{
	*w = 416; *h= 64;
}

void
RuleStateView::MouseDown(BPoint p)
{
	int i=Pt2Id(p);
	if(i<0) return;
	
	SetValue((uint32)Value()^((uint32)1<<i));
	Invoke();
}

void
RuleStateView::SetValue(int32 v)
{
	int32 old = Value();
	if(v==old) return;
	last_changed = ((uint32)v)^((uint32)old);
	BControl::SetValue(v);
}
void
RuleStateView::Draw(BRect r)
{
	if(button_image==NULL) return;
	SetDrawingMode(B_OP_COPY);
	uint32 v= Value();
	uint32 bit=1;
	for(int index=0; index<32; index++, bit<<=1)
	{
		if(bit&last_changed==0) continue;
		BRect s = r & cell_frame.OffsetByCopy(Id2Pt(index));
		if(!s.IsValid()) continue;
		if(v & bit) DrawBitmap(button_image, s.OffsetByCopy(0,64),s);
		else DrawBitmap(button_image, s,s);
	}
	last_changed = 0xffffffff;
}

static BPoint
Id2Pt(int id)
{
	return BPoint((id%16)*32, (id/16)*32);
}


static int
Pt2Id(BPoint p)
{
	if(FRAME.Contains(p))
		return int(p.x/32) + int(p.y/32) *16;
	else
		return -1;
}


status_t RuleStateView::Invoke(BMessage* msg)
{
	if(msg)
	{
		msg->AddInt32("rule", Value());
		return BControl::Invoke(msg);
	}
	if(Message())
	{
		BMessage m(Message());
		m.AddInt32("rule", Value());
		return BControl::Invoke(&m);
	}
	return BControl::Invoke(NULL);
}


/*
status_t RuleStateView::Invoke(BMessage* msg)
{
	Message()->ReplaceInt32("rule", Value());
	return BControl::Invoke(msg);
}




void
RuleStateView::MessageReceived(BMessage* msg)
{
}


	BRegion clip1, clip2;
	set_on_region(&clip1,v);
	ConstrainClippingRegion(&clip1);
	DrawBitmap(button_image, r.OffsetByCopy(0,64),r);
	clip2.Set(FRAME);
	clip2.Exclude(&clip1);
	ConstrainClippingRegion(&clip2);
	DrawBitmap(button_image, r,r);
	ConstrainClippingRegion(NULL);
static void set_on_region(BRegion* r, uint32 v)
{
	uint32 bit=1;
	for(int index=0; index<32; index++, bit<<=1)
	{
		if(v&bit)
			r->Include(cell_frame.OffsetByCopy(Id2Pt(index)));
	}
}
*/