
#include "SpeedSlider.h"
#include "MessageConstants.h"

SpeedSlider::SpeedSlider(BRect r)
	: BSlider(r,"speed", "Speed", NULL, 0, 100)
	, message_runner(NULL)
	, is_attached(false)
{
	SetHashMarks(B_HASH_MARKS_TOP);
	SetHashMarkCount(10+1);
	SetLimitLabels("Slow","Fast");
	BMessage* msg = new BMessage(MP_SET_SPEED);
	msg->AddInt32("speed", 0);
	msg->AddMessenger("reply", BMessenger());
	SetMessage(msg);
};
void
SpeedSlider::MessageReceived(BMessage* msg)
{
	switch(msg->what)
	{
	case MP_GET_FRAME_RATE:
		{
			float min, current, max;
			if(msg->FindFloat("min rate", &min)!=B_OK) min = 0;
			if(msg->FindFloat("max rate", &max)!=B_OK) max = 0;
			if(msg->FindFloat("current rate", &current)!=B_OK) current = 0;
			SetLimitLabels((BString()<<min).String(),(BString()<<max).String());
			SetLabel((BString()<<"Frame rates: "<<current<<" fps").String());
		}
		break;
	case MP_GET_SPEED:
		{
			int32 speed;
			if(msg->FindInt32("speed", &speed)==B_OK)
				SetValue(speed);
		}
	default:
		BSlider::MessageReceived(msg);
		return;
	}
}


void
SpeedSlider::AttachedToWindow()
{
	BSlider::AttachedToWindow();
	RestartMonitor();
	is_attached=true;
	Messenger().SendMessage(MP_GET_SPEED, this);
}
void
SpeedSlider::DetachedFromWindow()
{
	is_attached =false;
	StopMonitor();
	BSlider::DetachedFromWindow();
}

status_t
SpeedSlider::SetTarget(BMessenger messenger)
{
	StopMonitor();
	status_t st = BSlider::SetTarget(messenger);
	if(st==B_OK)
		if(is_attached) RestartMonitor();
	return st;
}

status_t
SpeedSlider::Invoke(BMessage* msg)
{
	Message()->ReplaceInt32("speed", Value());
	Message()->ReplaceMessenger("reply", BMessenger(this));
	return BSlider::Invoke(msg);
}

void
SpeedSlider::StopMonitor()
{
	if(message_runner)
	{
		delete message_runner;
		message_runner=NULL;
	}
}
void
SpeedSlider::RestartMonitor()
{
	StopMonitor();
	message_runner = new BMessageRunner(
		Messenger(),
		new BMessage(MP_GET_FRAME_RATE),
		1000*2000,
		-1,
		BMessenger(this, Looper()));
	
}
