#ifndef HEX_RULE_H
#define HEX_RULE_H

#include "Prefix.h"

typedef uint32 hex_rule;


enum NeighborIndex{
	iA0=0	,iA1,iA2,iB2,iC2,iA3,iB3,iC3,iA4,iB4,iC4,iA5,iA6,
	IA0=0x10,IA1,IA2,IB2,IC2,IA3,IB3,IC3,IA4,IB4,IC4,IA5,IA6};
enum NeighborState{
	nA0=(1<<iA0),
	nA1=(1<<iA1),
	nA2=(1<<iA2),
	nA3=(1<<iA3),
	nA4=(1<<iA4),
	nA5=(1<<iA5),
	nA6=(1<<iA6),
	nB2=(1<<iB2),
	nB3=(1<<iB3),
	nB4=(1<<iB4),
	nC2=(1<<iC2),
	nC3=(1<<iC3),
	nC4=(1<<iC4)
	};

enum ArithmeticState{
	aA0	=-1,
	aA1	=nA1|nA3|nB3|nC3|nA5,
	aA2	=nA2|nB3|nA4,
	aA3	=nA3|nB4|nA5,
	aA4	=nA4,
	aA5	=nA5,
	aA6	=nA6,
	aB2	=nB2|nA3|nB3|nC3|nB4,
	aB3	=nB3,
	aB4	=nB4,
	aC2	=nC2|nB3|nA4|nB4|nA6,
	aC3	=nC3|nB4|nA5,
	aC4	=nC4|nA5|nA6
	};
	
const hex_rule DEFAULT_RULE = (nA3|nA4|nA5|nA6)|((nA2|nB2|nC2)<<16);

const uint16 ARITHMETIC_MAP[16] =
	{aA0,aA1,aA2,aB2,aC2,aA3,aB3,aC3,aA4,aB4,aC4,aA5,aA6};

const uint16 ALTER_MAP[16] =
	{nA6,nA5,nA4,nB4,nC4,nA3,nB3,nC3,nA2,nB2,nC2,nA1,nA0};

uint32 ArithmeticRule(hex_rule rule);
hex_rule AlternateRule(hex_rule rule);
bool CanFlash(hex_rule rule);

//typedef uint32	UNIT;
//inline UNIT Result(UNIT,UNIT,UNIT,UNIT,UNIT,UNIT,UNIT, const UNIT*);

#define b3	x1
#define b4	x2
#define a3	x3
#define a4	x4
#define a5	x5
#define a6	x6

template<class UNIT>
inline UNIT Result(
	register UNIT x1,
	register UNIT x2,
	register UNIT x3,
	register UNIT x4,
	register UNIT x5,
	register UNIT x6,
	register UNIT C,
	const register UNIT*	rule
)
{
	register UNIT a1,a2,b2,c2,c3,c4;//,a3,a4,a5,a6,b3,b4;//
	c2 = (x1&x4)^(x2&x5)^(x3&x6);
	c3 = (x1&x3&x5)^(x2&x4&x6);
	c4 = ((~c2)&(x3&x6))^(x1&x2&x4&x5);
	a1 = x1^x3^x5;	b2 = x2^x4^x6;
	a2 = (a1&b2)^c2;
	a1 ^= b2;
	b2 = (x1|x3|x5)^(x2|x4|x6)^c3^a1;
	
	b3 = a1&a2;
	a3 = (a1&(a2^b2))^c3;
	a4 = a2&(b2^c2);
	b4 = c2&c3;
	a5 = a1&c4;
	a6 = c2&c4;

	UNIT live = 
			(		rule[iA0]) ^
			(a1&	rule[iA1]) ^
			(a2&	rule[iA2]) ^
			(b2&	rule[iB2]) ^
			(c2&	rule[iC2]) ^
			(a3&	rule[iA3]) ^
			(b3&	rule[iB3]) ^
			(c3&	rule[iC3]) ^
			(a4&	rule[iA4]) ^
			(b4&	rule[iB4]) ^
			(c4&	rule[iC4]) ^
			(a5&	rule[iA5]) ^
			(a6&	rule[iA6]);
		
	UNIT born = 
			(		rule[IA0]) ^
			(a1&	rule[IA1]) ^
			(a2&	rule[IA2]) ^
			(b2&	rule[IB2]) ^
			(c2&	rule[IC2]) ^
			(a3&	rule[IA3]) ^
			(b3&	rule[IB3]) ^
			(c3&	rule[IC3]) ^
			(a4&	rule[IA4]) ^
			(b4&	rule[IB4]) ^
			(c4&	rule[IC4]) ^
			(a5&	rule[IA5]) ^
			(a6&	rule[IA6]);

	return (born^((born^live)&C));
}
#undef b3
#undef b4
#undef a3
#undef a4
#undef a5
#undef a6

//			a0	= TRUE
//			a1 	= 1+2+3+4+5+6
//			a2 	= 12+23+34+45+56+61
//			b2 	= 13+24+35+46+51+62
//			c2 	= 14+25+36
//			a3 	= 123+234+345+456+561+612
//			b3l 	= 124+245+346+451+562+613
//			b3r 	= 125+236+341+452+563+614  
//			b3 	= b3l+b3r  
//			c3 	= 135+246
//			a4 	= 1234+2345+3456+4561+5612+6123
//			b4 	= 1235+2346+3451+4562+5613+6124
//			c4 	= 1245+2356+3461
//			a5 = 12345+23456+34561+45612+56123+61234
//			a6 = 123456

#endif

