#include "LifeHex.h"

#include "HexRule.h"
#include "HexGenerator.h"
#include "PixelMap.h"

static UNIT random_number();

LifeHex::LifeHex()
	: width(0)
	, height(0)
	, life_buffer(NULL)
	, life_buffer1(NULL)
	, life_buffer_size(0)
	, rule(DEFAULT_RULE)
	, generator(NULL)
	, init_check(false)
{
	generator = new HexGenerator(DEFAULT_RULE);
}


LifeHex::~LifeHex()
{
	Dispose();
	delete 	generator;
}


void
LifeHex::Dispose()
{
	if(init_check)
	{
		life_buffer_size = 0;
		delete_area(area); area=-1;
		life_buffer = NULL;life_buffer1=NULL;
		width=0;
		height=0;
		init_check = false;
	}
}

void
LifeHex::Generate()
{
	if(init_check)
	{
		generator->Generate(
			(UNIT*)life_buffer, 
			(UNIT*)life_buffer1, 
			life_buffer_size/sizeof(UNIT), 
			width/(sizeof(UNIT)*8));
		void* v= life_buffer; life_buffer = life_buffer1; life_buffer1=v;
	}
}

	static UNIT random_unit()
	{
		UNIT val;
		uint8* a=(uint8*)(&val);
		for(int i=0;i<sizeof(UNIT);i++) a[i]= random();
		return val;
	}

void
LifeHex::RandomizeLocal(int diameter)
{
	if(!init_check) return;

	const int BPU(sizeof(UNIT)*8);
	const UNIT MASK(-1);
	int radius = diameter/2;
	if(radius<=1) radius=1;
	if(radius>=BPU/2) radius = BPU/2-1;
	diameter = diameter/2*2+1;
	
	PixelMap m = GetBufferInfo();
	int x0 = random_unit()%(m.width);
	int y0 = random_unit()%(m.height) + radius;
	
	for(int i =-radius; i<=radius;i++)
	{
		UNIT* u(NULL);
		m.GetAddress(&u, x0,  (y0+i) % m.height);
		UNIT mask = (i<0)?(MASK>>-i):(MASK<<i);
		*u = ((*u)&(~mask))|(random_unit()&mask);
	}
}

void
LifeHex::Randomize()
{
	if(!init_check) return;
	srandom(system_time());
	uint8 *array = (uint8*)life_buffer;
	for(int i=0;i<life_buffer_size;i++) array[i] = random();
}

void
LifeHex::SetRule(hex_rule r)
{
	rule=r;
	generator->SetRule(r);
}

hex_rule
LifeHex::Rule() const
{
	return rule;
}

void
LifeHex::SetSize(int w, int h)
{
	Dispose();

	const int u = sizeof(UNIT)*8;
	{
		width = w/u*u;
		height = h/u*u;
		if(width<=0) width = u;
		if(height<=0) height = u;
	}

	life_buffer_size = width * (height+2) / 8;
	size_t area_size = life_buffer_size*2;
	area_size = (area_size+B_PAGE_SIZE-1)/B_PAGE_SIZE*B_PAGE_SIZE;
	area = create_area(
		"life_buffer",
		&life_buffer, 
		B_ANY_ADDRESS,
		area_size,
		B_FULL_LOCK,
		B_READ_AREA|B_WRITE_AREA
	);
	if(area<B_OK){
		life_buffer_size=0;
		width =0; height=0;
		return;
	}
	life_buffer1 = (char*)life_buffer + life_buffer_size;

	init_check = true;
}

void
LifeHex::GetSize(int* w, int* h) const
{
	*w = width; *h=height;
}

void*
LifeHex::GetBuffer() const
{
	return life_buffer;
}

bool LifeHex::InitCheck() const
{
	return init_check;
}

PixelMap
LifeHex::GetBufferInfo() const
{
	return PixelMap(life_buffer, 0, width,width,height,1);
}
