#include "LifeApplication.h"

#include "LifeLooper.h"
#include "LifeControlWindow.h"
#include "RuleControlWindow.h"
#include "RuleListWindow.h"

const char* LifeApplication::SIGNATURE = 
	"application/x-vnd.LifeHex-YukioHirose";

	static class RuleFileFilter:public BRefFilter
	{
		bool Filter(const entry_ref*, BNode* node, struct stat*, const char* file_type)
		{
			return (
				node->IsDirectory()||
				node->IsSymLink()||
				(strncmp(file_type, "text", 4)==0)
			);
		}
	};

int main(int argc, char* argv[])
{
	BApplication* my_app = new LifeApplication();
	my_app->Run();
	delete my_app;
	return 0;
}


LifeApplication::LifeApplication()
	: BApplication(SIGNATURE)
	, life_looper(NULL)
	, control_window(NULL)
	, rule_control_window(NULL)
	, open_panel(NULL)
	, ref_filter(NULL)
	, ref(NULL)
{
	life_looper = new LifeLooper();
	control_window = new LifeControlWindow
		( BPoint(540,30)
		, BMessenger(NULL, life_looper)
		);
	rule_control_window = new RuleControlWindow
		( BPoint(20,30)
		, BMessenger(NULL, life_looper)
		);
	ref = new entry_ref;
	ref_filter = new RuleFileFilter();
}


LifeApplication::~LifeApplication()
{
	life_looper->Lock();
	life_looper->Quit();
}



void
LifeApplication::MessageReceived(BMessage* msg)
{
	switch(msg->what)
	{
	case MA_OPEN:
		if(open_panel==NULL) open_panel = new BFilePanel();
		open_panel->SetRefFilter(ref_filter);
		open_panel->SetPanelDirectory(ref);
		open_panel->Show();
		break;
	case MA_NEW:
		PostMessage(B_REFS_RECEIVED);
		break;
	case B_CANCEL:
		if(open_panel)
		{
			open_panel->GetPanelDirectory(ref);
			delete open_panel;
			open_panel = NULL;
		}
		break;
	default:
		BApplication::MessageReceived(msg);
	}
}

bool
LifeApplication::QuitRequested()
{
	BWindow* w;
	while(true)
	{
		{
			BAutolock l(RuleListWindow::window_list_lock);
			if(!l.IsLocked()) break;
			w = (BWindow*)(RuleListWindow::window_list.LastItem());
			if(w==NULL) break;
			w->Lock();
		}
		if(!w->QuitRequested())
		{
			w->Unlock();
			return false;
		}
		w->Quit();
	}
	control_window->Lock();
	control_window->Quit();
	rule_control_window->Lock();
	rule_control_window->Quit();
	return true;
}



void
LifeApplication::ReadyToRun()
{
	life_looper->Run();
	control_window->Show();
	rule_control_window->Show();
}
void
LifeApplication::RefsReceived(BMessage *msg)
{
	BWindow* rule_list_window = new RuleListWindow(
		BRect(540,200,800,600),
		BMessenger(NULL, rule_control_window));
	rule_list_window->Show();
	BMessage m(msg);
	rule_list_window->PostMessage(&m);
}


void
LifeApplication::AboutRequested()
{
	BAlert* alert = new BAlert
		( "About Alert"
		, " LifeHex© by Yukio Hirose.\npeechan@po.aianet.ne.jp"
		, "OK"
	);
	alert->Go();
}






