#include "LifeLooper.h"

#include "LifeDaemon.h"
#include "RenderHandler.h"
#include "ScreenHandler.h"
#include "GenerationHandler.h"
#include "MessageConstants.h"


LifeLooper::LifeLooper()
	: BLooper("life looper", B_NORMAL_PRIORITY)
	, daemon(NULL)
	, render_handler(NULL)
	, generation_handler(NULL)
{
	daemon = new LifeDaemon();

	render_handler = new RenderHandler(daemon);
	AddHandler(render_handler);

	generation_handler = new GenerationHandler(daemon);
	AddHandler(generation_handler);

	screen_handler = new ScreenHandler(daemon);
	AddHandler(screen_handler);

	screen_handler->SetNextHandler(generation_handler);
	generation_handler->SetNextHandler(render_handler);
	SetPreferredHandler(screen_handler);

//	PostMessage(MP_PLAY, this);
	PostMessage(MG_RANDOMIZE, generation_handler);
	PostMessage(MS_WINDOW, screen_handler);

	daemon->Resume();
}

LifeLooper::~LifeLooper()
{
	daemon->Lock();
	delete daemon;
	
	RemoveHandler(screen_handler);
	RemoveHandler(generation_handler);
	RemoveHandler(render_handler);
	
	delete screen_handler;
	delete render_handler;
	delete generation_handler;
}

void
LifeLooper::DispatchMessage(BMessage* message, BHandler* handler)
{
//	if(message->IsSourceWaiting())...
	daemon->Lock();
	BLooper::DispatchMessage(message, handler);
	daemon->Unlock();
}

void
LifeLooper::MessageReceived(BMessage* msg)
{
	switch(msg->what)
	{
	case MP_PLAY:
		daemon->Go();
		break;
	case MP_STEP:
		daemon->Step();
		break;
	case MP_SET_PERIOD:
		{
			int32 period;
			if(msg->FindInt32("period", &period)==B_OK)
				daemon->SetPeriod(period);
		}
		break;
	case MP_GET_PERIOD:
		{
			BMessage message(MP_GET_PERIOD);
			message.AddInt32("period", daemon->Period());
			msg->SendReply(&message);
		}
		break;
	case MP_SET_SPEED:
		{
			int32 speed;
			if(msg->FindInt32("speed", &speed)==B_OK)
			daemon->SetSpeed(speed);
		}
		break;
	case MP_GET_SPEED:
		{
			BMessage message(MP_GET_SPEED);
			message.AddInt32("speed", daemon->Speed());
			msg->SendReply(&message);
		}
		break;
	case MP_SET_AVOID_FLASH:
		{
			int32 v;
			bool b;
			if(msg->FindInt32("value", &v)!=B_OK) b=true;
			else b= (v==B_CONTROL_ON);
			daemon->SetAvoidFlash(b);
		}
		break;
	case MP_GET_FRAME_RATE:
		{
//			BMessenger m = ReturnAddress();
//			if(!m.IsValid());
			BMessage message(MP_GET_FRAME_RATE);
			float min, current, max;
			daemon->GetFrameRates(&min,&current, &max);
			message.AddFloat("max rate", max);
			message.AddFloat("min rate", min);
			message.AddFloat("current rate", current);
			msg->SendReply(&message);
		}
		break;
	case MP_SET_PRIORITY:
		{
			int32 period;
			if(msg->FindInt32("priority", &period)==B_OK)
				daemon->SetPriority(period);
		}
		break;
	case MP_GET_PRIORITY:
		{
			BMessage message(MP_GET_PRIORITY);
//			message.AddInt32("priority", daemon->GetPriority());
			msg->SendReply(&message);
		}
		break;
	default:
		BLooper::MessageReceived(msg);
		return;
	}
}

