#include "RenderHandler.h"

#include "MessageConstants.h"
#include "LifeDaemon.h"
#include "BufferedRenderer.h"
#include "LifeRenderer.h"
#include "RenderRow_1_4.h"
#include "HexRendererF8.h"
//#include "HexRendererI8.h"
//#include "RenderRowMono8.h"



RenderHandler::RenderHandler(LifeDaemon* d)
	: BHandler("RenderHandler")
	, render_option(GRADATION)
	, scale_option(SCALE_2)
	, renderer(NULL)
	, daemon(d)
{}

RenderHandler::~RenderHandler()
{
	daemon->renderer=NULL;
	if(renderer) delete renderer;
}


void
RenderHandler::MessageReceived(BMessage* msg)
{
	switch(msg->what)
	{
	case MR_SET_SCALE:
		{
			int32 scale;
			if(msg->FindInt32("scale", &scale)==B_OK)
			{
				SetScaleOption((ScaleOption)scale);
				Reset();
			}
		}
		break;
	case MR_SET_METHOD:
		{
			int32 index;
			if(msg->FindInt32("index", &index)==B_OK)
			{
				SetMethodOption((MethodOption)index);
				Reset();
			}
		}
		break;
	case MC_GET_VALUES:
		{
			int32 l;
			if(msg->FindInt32("length", &l) !=B_OK) l=256;
			if(l<=0||l>256) return;

			BMessage m(MC_GET_VALUES);
			m.AddData("colors", B_RGB_COLOR_TYPE, colors, 
				sizeof(rgb_color)*256);
			msg->SendReply(&m);
		}
		break;
	case MC_SET_VALUES:
		{
			const rgb_color* data;
			ssize_t size=0;
			msg->FindData("colors",B_RGB_COLOR_TYPE, (const void **)(&data), &size);
			size /= sizeof(rgb_color);
			size = min_c(256, size);
			for(int i=0;i<size;i++)
				colors[i]= data[i];
			if(renderer)renderer->ColorModelChanged();
		}
		break;
	case MC_SET_VALUE:
		{
			int32 data;
			if(msg->FindInt32("color", &data)==B_OK)
			{
				int i = data&0xff;
				colors[i].red = (data>>24);
				colors[i].green = (data>>16);
				colors[i].blue = (data>>8);
				colors[i].alpha = 0x80;
			}
			ResetColorData();
			if(renderer)renderer->ColorModelChanged();
		}
		break;
		default:BHandler::MessageReceived(msg);
		break;
	}
}

void
RenderHandler::SetOptions(MethodOption c, ScaleOption s)
{
	render_option = c;
	scale_option = s;
	Reset();
}



void
RenderHandler::Reset()
{
	daemon->renderer = NULL;	
	if(renderer) delete renderer;
	renderer = GetRenderer();
	daemon->renderer = renderer;
}


Renderer*
RenderHandler::GetRenderer()
{
	if(true)
	{
		Renderer* r(NULL);
		RenderRowFunction* f(NULL);
		switch(render_option)
		{
			case MONOCHROME:
					f = new RenderRow_1_4M();break;
			case ROTATION:
					f = new RenderRow_1_4R();break;
			case GENERATION:
					f = new RenderRow_1_4G();break;
			case GRADATION:
			default:
					f = new RenderRow_1_4GR();break;
		}
		switch(scale_option)
		{
			case SCALE_2:
					r = new HexRendererF8_2();break;
			case SCALE_4:
					r = new HexRendererF8_4();break;
			case SCALE_8:
					r = new HexRendererF8_8();break;
			case SCALE_1:
			default:
					r = new HexRendererF8_1();break;
		}
		return new BufferedRenderer(
			new LifeRenderer(f), r, 4);
	}


}

extern uint8 color_dataF8[16];

void
RenderHandler::ResetColorData()
{
	BScreen s(B_MAIN_SCREEN_ID);
	for(int i=0;i<16;i++)
		color_dataF8[i]= s.IndexForColor(colors[i]);
}

