#ifndef BIT_ITERATOR_H
#define BIT_ITERATOR_H



template <class UNIT>
class BitIterator8
{
private:
	static const int BPU = sizeof(UNIT)*8;
	static const int PERIOD = 8;
	static const int MASK = 0xff;
	const UNIT* current;
	int shift;
	UNIT value;
	
public:
	BitIterator8(const UNIT* const bits, int offset)
		: current(NULL)
		, shift(0)
	{
		reset(bits, offset);
	}

	UNIT next()
	{
		shift-= PERIOD;
		if(shift>=0)
			return (value>>shift)&MASK;
		else
		{
			UNIT v = value << (-shift);
			shift+=BPU;
			value = *(++current);
			return (v|(value>>shift))&MASK;
		}
	}
	void skip(int offset)
	{
		reset(current, offset+ BPU-shift);
	}
	void reset(const UNIT* const bits, int offset)
	{
		current = bits+offset/BPU;
		shift = BPU-(offset%BPU);
		if(shift == BPU)
		{
			shift=0;
			current--;
			value=0;
		}
		else
			value = *current;
	}
};


#endif

/*
#define BPU (sizeof(UNIT)*8)
#define LOOP_mask(period)  ((1<<(period)-1)
//#define LOOP_MASK_ODD  ((1<<(period+1)-1)

#define BEGIN_LOOP(bits, offset, period, count) {\
	int LOOP_shift(BPU-(offset)%BPU);\
	UNIT* LOOP_pt((bits)+(offset)/BPU);\
	UNIT LOOP_val;\
	while(count-->0){
#define END_LOOP \
		}
	}
#define NEXT (\
	LOOP_shift-=period, \
	(LOOP_shift>=0)?\
	((LOOP_val>>LOOP_shift)&LOOP_mask(period)):\
	(LOOP_shift+=BPU, LOOP_val = *(++LOOP_pt), (LOOP_val>>LOOP_shift) &LOOP_mask(period))\
	)

*/