#include "Maps.h"


PixelMap GetPixelMap(const PixelMap* p, int32 l, int32 t, int32 r, int32 b)
{
	return PixelMap(p->bits, p->BitsOffset(l,t), p->bits_per_row, r-l, t-b, p->pixel_size);
}


	
PixelMap GetPixelMap(const BBitmap* s)
{
	int pixel_size;
	switch(s->ColorSpace())
	{
	case B_GRAY1:		pixel_size = 1; break;
	case B_GRAY8:		pixel_size = 8; break;
	case B_CMAP8:		pixel_size = 8; break;
	case B_RGBA15_BIG:	pixel_size = 16; break;
	case B_RGB15_BIG:	pixel_size = 16; break;
	case B_RGB16_BIG:	pixel_size = 16; break;
//		case B_RGB24_BIG:	pixel_size = BBB; break;
	case B_RGBA32_BIG:	pixel_size = 32; break;
	case B_RGB32_BIG:	pixel_size = 32; break;
	case B_RGB15:		pixel_size = 16; break;
	case B_RGB16:		pixel_size = 16; break;
//		case B_RGB24:		pixel_size = BBB; break;
	case B_RGBA32:		pixel_size = 32; break;
	case B_RGB32:		pixel_size = 32; break;
	default:			pixel_size=1;
	}
	
	return PixelMap(
		s->Bits(),
		0,
		s->BytesPerRow()*8,
		s->Bounds().IntegerWidth()+1,
		s->Bounds().IntegerHeight()+1,
		pixel_size
		);
}

/*


class ConcreteMap:public PixelMap
{
public:
	ConcreteMap(int width, int height, int pixel_sixe);
	~ConcreteMap();
		
	int		BitsOrigin(void** base) const
									{*base = bits;return 0;}
	int		BitsPerRow() const		{return bits_per_row;}
	int		PixelSize() const		{return pixel_size;}
	int		Width() const			{return width;}
	int		Height() const			{return height;}
		
protected:
	void*	bits;
	int		width;
	int		height;
	int		pixel_size;
	int		bits_per_row;
	area_id	area;
};


class BitMap:public PixelMap
{
public:
			BitMap(BBitmap* source);
			~BitMap();

	int		BitsOrigin(void** base) const
									{*base = source->Bits();return 0;}
	int		BitsPerRow() const		{return source->BytesPerRow()*8;}
	int		PixelSize() const		{return pixel_size;}
	int		Width() const			{return source->Bounds().IntegerWidth()+1;}
	int		Height() const			{return source->Bounds().IntegerHeight()+1;}

protected:
	BBitmap* source;
	int pixel_size;
};


LocalMap::LocalMap(const PixelMap* parent)
	: PixelMap()
	, parent(parent)
{
	SetFrame(0,0,parent->Width(), parent->Height());
}

LocalMap::LocalMap(const PixelMap* parent, int32 left, int32 top, int32 right, int32 bottom)
	: PixelMap()
	, parent(parent)
{	SetFrame(left, top, right, bottom);}

void LocalMap::SetFrame(int32 l, int32 t, int32 r, int32 b)
{
	left = max_c(l, 0);
	top = max_c(t, 0);
	right = min_c(r, parent->Width());
	bottom = min_c(b, parent->Height());
}

ConcreteMap::ConcreteMap(int w, int h, int s)
	: bits(NULL)
	, width(max_c(w, 1))
	, height(max_c(h, 1))
	, pixel_size(max_c(s, 1))
	, area(-1)
{
	int l = width*pixel_size;
	l--;l/=32;l++;
	bits_per_row = l*32;
//	bits = new uint32[l*height];

	size_t area_size = bits_per_row * height / 8;
	area_size = (area_size+B_PAGE_SIZE-1)/B_PAGE_SIZE*B_PAGE_SIZE;
	area = create_area(
		"concrete map",
		&bits, 
		B_ANY_ADDRESS,
		area_size,
		B_FULL_LOCK,
		B_READ_AREA|B_WRITE_AREA
	);
	if(area<B_OK){
		bits=NULL;
		return;
	}
}

ConcreteMap::~ConcreteMap()
{
	if(area>=B_OK) delete_area(area);
}

BitMap::~BitMap(){}



*/