#include "Prefix.h"

#include <ctype.h>



bool ReadRule(const BString& source, int32* offset, uint32* rule, BString* name);
void WriteRule(BString* dest, uint32 rule, const char* name); 

status_t ReadString(BDataIO* f, BString* s, size_t limit);
status_t WriteString(BDataIO* f, BString* s);

BString Rule2String(uint32);
uint32 String2Rule(const BString&);


#include "HexRule.h"

ssize_t ReadString(BDataIO* f, BString* s, size_t limit)
{
	const size_t LENGTH = 1024;
	char buffer[LENGTH];
	size_t num_read=0;
	ssize_t read;

	while(true)
	{
		read = f->Read(buffer, LENGTH);
		if(read<B_OK) return read;
		s->Append(buffer, read);
		num_read += read;
		if((read<LENGTH)||(num_read>=limit)) return num_read;
	}
}

ssize_t WriteString(BDataIO* f, BString* s)
{
	return f->Write(s->String(), s->Length());
}

bool ReadRule(const BString& source, int32* offset, uint32* rule, BString* name)
{
	BString rstr;
	int L=source.Length();
	int32 t0,t1,t2;
	if(*offset>=L) return false;

	t1 = source.FindFirst('\t', *offset);
		if(t1<0) {*offset=L;return false;}
	t0 = source.FindLast('\n', t1);
		if(t0<0)t0=-1;
	t2 = source.FindFirst('\n', t1);
		if(t2<0) {*offset=L;return false;}
	*offset = t2+1;

	source.CopyInto(*name, t0+1, min_c(63, t1-(t0+1)));
	source.CopyInto(rstr, t1+1, min_c(63, t2-(t1+1)));
	*rule = String2Rule(rstr);
	return true;
}


void WriteRule(BString* dest, uint32 rule, const char* name)
{
	char rule_string[64];
	
	*dest<<name;
	*dest<<'\t';
	*dest<<Rule2String(rule);
	*dest<<'\n';
}



uint32 String2Rule(const BString& source)
{
	static const uint32 aa[7]={nA0,nA1,nA2,nA3,nA4,nA5,nA6};
	static const uint32 bb[7]={0,0,nB2,nB3,nB4,0,0};
	static const uint32 cc[7]={0,0,nC2,nC3,nC4,0,0};
	uint32 rule(0);
	char c;
	int i=0;
	const int L=source.Length();
	const uint32* array = NULL;
	bool is_born;
	while(i<L)
	{
		c = source[i++];
		if(!isxdigit(c)){array = NULL;continue;}
		if((array!=NULL) && (c>='0') && (c<='6'))
		{
			int i= c-'0';
			rule|= array[i]<< (is_born? 16:0);
			continue;
		}
		switch(toupper(c))
		{
		case 'A':array = aa;break;
		case 'B':array = bb;break;
		case 'C':array = cc;break;
		default:
			array=NULL;
			continue;
		}
		is_born = isupper(c);
	}
	return rule;
}

BString Rule2String(uint32 source)
{
	BString dest;
	static const uint32 BITS[13]=
		{nA0,nA1,nA2,nA3,nA4,nA5,nA6,nB2,nB3,nB4,nC2,nC3,nC4,};
	static const uint32 AA=nA0|nA1|nA2|nA3|nA4|nA5|nA6;
	static const uint32 BB=nB2|nB3|nB4;
	static const uint32 CC=nC2|nC3|nC4;
	
	bool first=true;
LOOP:
	{
		if(AA&source)
		{
			dest<<'A';
			for(int i=0;i<7;i++)
				if(BITS[i]&source) dest<<char('0'+i);
		}
		if(BB&source)
		{
			dest<<'B';
			for(int i=7;i<10;i++)
				if(BITS[i]&source) dest<<char('0'+(i-5));
		}
		if(CC&source)
		{
			dest<<'C';
			for(int i=10;i<13;i++)
				if(BITS[i]&source) dest<<char('0'+(i-8));
		}
	}
	if(first)
	{
		first=false;
		dest.ToLower();
		dest<<' ';
		source>>=16;
		goto LOOP;
	}
	return dest;
}




