#include "BufferedRenderer.h"

#include "Maps.h"


BufferedRenderer::BufferedRenderer(Renderer* rendererS, Renderer* rendererD, int pixel_size)
	: Renderer(rendererS->ScaleX()*rendererD->ScaleX(), rendererS->ScaleY()*rendererD->ScaleY())
	, pixel_size(pixel_size)
	, rendererS(rendererS)
	, rendererD(rendererD)
	, area(-1)
{
	buffer.pixel_size=0;
}

BufferedRenderer::~BufferedRenderer()
{
	delete rendererS;
	delete rendererD;
	if(area>=B_OK) delete_area(area);
}

void
BufferedRenderer::ColorModelChanged()
{
	if(rendererS) rendererS->ColorModelChanged();
	if(rendererD) rendererD->ColorModelChanged();
}

void BufferedRenderer::Render(const PixelMap* source, PixelMap* dest)
{
	if(buffer.pixel_size==0)
		InitBuffer(source, dest);
	if(buffer.pixel_size<=0) return;
	rendererS->Render(source, &buffer);
	rendererD->Render(&buffer, dest);
}

void BufferedRenderer::InitBuffer(const PixelMap* source, const PixelMap* dest)
{
	int w = min_c(source->width*rendererS->ScaleX(), dest->width/rendererD->ScaleX());
	int h = min_c(source->height*rendererS->ScaleY(), dest->height/rendererD->ScaleY());
	
	int ww = w*pixel_size;
	ww = (ww-1)/32*32 + 32;
	w = ww/pixel_size;
	
	PixelMap m(NULL,0, ww, ww/pixel_size, h, pixel_size);

	size_t area_size = m.bits_per_row * m.height/8;
	area_size = (area_size+B_PAGE_SIZE-1)/B_PAGE_SIZE*B_PAGE_SIZE;
	area = create_area("life_hex render middle buffer",&(m.bits), B_ANY_ADDRESS, area_size,
		B_FULL_LOCK,B_READ_AREA|B_WRITE_AREA);
		
	if(area>=B_OK) buffer = m;
}

/*

void BufferedRenderer::InitParams(const PixelMap* source, const PixelMap* dest)
{
	if(buffer == NULL)
	{
		int sw = source->Width();
		int sh = source->Height();
		int dw = dest->Width()/rendererD->Scale().x;
		int dh = dest->Height()/rendererD->Scale().y;
		
		int	w = (sw<dw)? sw :dw; w = (w-1)/64*64 + 64;
		int h = (sh<dh)? sh :dh;
		buffer = new ConcreteMap(w, h, pixel_size);
	}
	init_ok = (buffer!=NULL);
	if(init_ok)
	{
		rendererS->InitParams(source, buffer);
		rendererD->InitParams(buffer, dest);
	}
}

*/
