#include "HexRendererF8.h"


uint8 color_dataF8[16]={0,1,2,3,4,5,6,7,8,9,0xa,0xb,0xc,0xd,0xe,0xf};


/***********************************************************************************/
HexRendererF8_1::HexRendererF8_1()
	:HexRenderer(1,1)
{
	ColorModelChanged();
}
void
HexRendererF8_1::ColorModelChanged()
{
	for(uint32 i=0;i<256;i++)
	{
		uint32 c = color_dataF8[i>>4]<<8;
		c|= color_dataF8[i&0xf];
		data1[i] = c<<16;
		data2[i] = c;
	}
	swap_data(B_INT32_TYPE, data1, 256*sizeof(int32), B_SWAP_BENDIAN_TO_HOST);
	swap_data(B_INT32_TYPE, data2, 256*sizeof(int32), B_SWAP_BENDIAN_TO_HOST);
}

void
HexRendererF8_1::render_even(void* source, void* dest, int count, int)
{
	uint8*	s((uint8*)source);
	uint8*	e(s + count/2);
	uint32*	d((uint32*)dest);
	while(s<e)
	{
		*(d++) = data1[*(s++)]|data2[*(s++)];
	}
}

void
HexRendererF8_1::render_odd(void* source, void* dest, int count, int)
{
	render_even(source, dest, count,0);
}

/***********************************************************************************/
HexRendererF8_2::HexRendererF8_2()
	:HexRenderer(2,2)
{ColorModelChanged();}

void
HexRendererF8_2::ColorModelChanged()
{
	for(uint32 i=0;i<256;i++)
	{
		data[i] = uint32(color_dataF8[i&0xf])|(uint32(color_dataF8[i>>4])<<16);
		data[i] |= data[i]<<8;
//		dataH[i]=data[i]>>8;
	}
	swap_data(B_INT32_TYPE, data, 256*sizeof(int32), B_SWAP_BENDIAN_TO_HOST);
}

void
HexRendererF8_2::render_even(void* source, void* dest, int count, int bpr)
{
	uint8*	s((uint8*)source);
	uint8*	e(s + count/2);
	uint32*	d0((uint32*)dest);
	uint32*	d1(d0 + bpr/32);
	while(s<e)
	{
		*(d0++) = *(d1++) = data[*(s++)];
	}
}
void
HexRendererF8_2::render_odd(void* source, void* dest, int count, int bpr)
{
	uint8*	s((uint8*)source);
	uint8*	e(s + count/2);
	uint32*	d0((uint32*)dest);
	uint32*	d1(d0 + bpr/32);
	register uint32	v(0); //(data[*(s-1)]);
	
	while(s<e)
#if LITTLE_ENDIAN
		*(d0++) = *(d1++) = (v = (v>>24)|(data[*(s++)]<<8));
#else
		*(d0++) = *(d1++) = (v = (v<<24)|(data[*(s++)]>>8));
#endif
}
/***********************************************************************************/

HexRendererF8_4::HexRendererF8_4()
	:HexRenderer(4,4){ColorModelChanged();}

void
HexRendererF8_4::ColorModelChanged()
{
	for(uint32 i=0;i<256;i++)
	{
		data1[i] = color_dataF8[i>>4]*0x01010101;
		data2[i] = color_dataF8[i&0xf]*0x01010101;
	}
}

void
HexRendererF8_4::render_even(void* source, void* dest, int count, int bpr)
{
	uint8*		s((uint8*)source);
	uint8*		e(s + count/2);
	bpr/=32;
	uint32*	d0((uint32*)dest);
	uint32*	d1(d0 + bpr);
	uint32*	d2(d1 + bpr);
	uint32*	d3(d2 + bpr);
	uint32	v;
	while(s<e)
	{
		v = data1[*s];
		*(d3++) = *(d0++) = v&0x00ffff00;
		*(d2++) = *(d1++) = v;
		v = data2[*(s++)];
		*(d3++) = *(d0++) = v&0x00ffff00;
		*(d2++) = *(d1++) = v;
	}
}


void
HexRendererF8_4::render_odd(void* source, void* dest, int count, int bpr)
{
	uint8*		s((uint8*)source);
	uint8*		e(s + count/2);
	bpr/=32;
	uint32*	d0((uint32*)dest);
	uint32*	d1(d0 + bpr);
	uint32*	d2(d1 + bpr);
	uint32*	d3(d2 + bpr);
	register uint32 v;// (data2[*(s-1)])
	while(s<e)
	{
#if LITTLE_ENDIAN
		v = (v>>16)|(data1[*s]<<16);
		*(d3++) = *(d0++) = v&0xff0000ff;
		*(d2++) = *(d1++) = v;
		v = (v>>16)|(data2[*(s++)]<<16);
		*(d3++) = *(d0++) = v&0xff0000ff;
		*(d2++) = *(d1++) = v;
#else
		v = (v<<16)|(data1[*s]>>16);
		*(d3++) = *(d0++) = v&0xff0000ff;
		*(d2++) = *(d1++) = v;
		v = (v<<16)|(data2[*(s++)]>>16);
		*(d3++) = *(d0++) = v&0xff0000ff;
		*(d2++) = *(d1++) = v;
#endif
	}
}
/***********************************************************************************/


HexRendererF8_8::HexRendererF8_8()
	:HexRenderer(8, 7){ColorModelChanged();}
	
void
HexRendererF8_8::ColorModelChanged()
{
	for(uint32 i=0;i<16;i++)
	{
		data[i] = color_dataF8[i]*0x01010101;
	}
}

#if LITTLE_ENDIAN
	#define MASK_0A 0xffff0000
	#define MASK_0B 0x000000ff
	#define MASK_1A 0xffffff00
	#define MASK_1B 0x0000ffff
	#define MASK_2A 0xffffffff
	#define MASK_2B 0x00ffffff
#else
	#define MASK_0A 0x0000ffff
	#define MASK_0B 0xff000000
	#define MASK_1A 0x00ffffff
	#define MASK_1B 0xffff0000
	#define MASK_2A 0xffffffff
	#define MASK_2B 0xffffff00
#endif
void
HexRendererF8_8::render_even(void* source, void* dest, int count, int bpr)
{
	uint8*	s((uint8*)source);
	uint8*	e(s + count/2);
	bpr/=32;
	uint32*	d0((uint32*)dest);
	uint32*	d1(d0 + bpr);
	uint32*	d2(d1 + bpr);
	uint32*	d3(d2 + bpr);
	uint32*	d4(d3 + bpr);
	uint32*	d5(d4 + bpr);
	uint32*	d6(d5 + bpr);
	uint32	v;
	while(s<e)
	{
		v = data[*s>>4];
		*(d0++) = *(d6++) = v&MASK_0A;
		*(d0++) = *(d6++) = v&MASK_0B;
		*(d1++) = *(d5++) = v&MASK_1A;
		*(d1++) = *(d5++) = v&MASK_1B;
		*(d2++) = *(d3++) = *(d4++) = v;
		*(d2++) = *(d3++) = *(d4++) = v&MASK_2B;
		v = data[*(s++)&0xf];
		*(d0++) = *(d6++) = v&MASK_0A;
		*(d0++) = *(d6++) = v&MASK_0B;
		*(d1++) = *(d5++) = v&MASK_1A;
		*(d1++) = *(d5++) = v&MASK_1B;
		*(d2++) = *(d3++) = *(d4++) = v;
		*(d2++) = *(d3++) = *(d4++) = v&MASK_2B;
	}
}
void
HexRendererF8_8::render_odd(void* source, void* dest, int count, int bpr)
{
	uint8*	s((uint8*)source);
	uint8*	e(s + count/2);
	bpr/=32;
	uint32*	d0((uint32*)dest);
	uint32*	d1(d0 + bpr);
	uint32*	d2(d1 + bpr);
	uint32*	d3(d2 + bpr);
	uint32*	d4(d3 + bpr);
	uint32*	d5(d4 + bpr);
	uint32*	d6(d5 + bpr);
	uint32	v(0);//(data[(*(s-1))&0xf]);
	while(s<e)
	{
		*(d0++) = *(d6++) = v&MASK_0B;
		*(d1++) = *(d5++) = v&MASK_1B;
		*(d2++) = *(d3++) = *(d4++) = v&MASK_2B;
		v = data[*s>>4];
		*(d0++) = *(d6++) = v&MASK_0A;
		*(d1++) = *(d5++) = v&MASK_1A;
		*(d2++) = *(d3++) = *(d4++) = v;

		*(d0++) = *(d6++) = v&MASK_0B;
		*(d1++) = *(d5++) = v&MASK_1B;
		*(d2++) = *(d3++) = *(d4++) = v&MASK_2B;
		v = data[*(s++)&0xf];
		*(d0++) = *(d6++) = v&MASK_0A;
		*(d1++) = *(d5++) = v&MASK_1A;
		*(d2++) = *(d3++) = *(d4++) = v;
	}
}

