
#include "LifeRenderer.h"
#include "PixelMap.h"
#include "RenderRowFunction.h"

LifeRenderer::LifeRenderer()
	: Renderer(1,1)
	, even_func(NULL)
	, odd_func(NULL)
{
}


LifeRenderer::LifeRenderer(RenderRowFunction* even, RenderRowFunction* odd)
	:Renderer(1,1)
	, even_func(NULL)
	, odd_func(NULL)
{
	SetFunctions(even, odd);
}

LifeRenderer::~LifeRenderer()
{
	SetFunctions(NULL);
}

void
LifeRenderer::SetFunctions(RenderRowFunction* even, RenderRowFunction* odd)
{
	{
		if(even_func==odd_func) odd_func=NULL;
		if(even_func) delete even_func;
		if(odd_func) delete odd_func;
		scale_x=0;scale_y=0;
	}
	if(even==NULL) return;
	even_func = even;
	if(odd==NULL) odd=even;
	odd_func = odd;
	scale_x = even->scale_x;
	scale_y = even->scale_y;
}

void LifeRenderer::Render(const PixelMap* s_map, PixelMap* d_map)
{
	if((s_map->pixel_size!=even_func->source_pixel_size) ||
		 (d_map->pixel_size!=even_func->dest_pixel_size))return;
	
	int w = min_c(s_map->width, d_map->width/scale_x);
	int h = min_c(s_map->height, d_map->height/scale_y);
	
	const int total = s_map->bits_per_row* s_map->height;
	const int base = s_map->bits_offset;

	int offset = base;
	uint32 *dest_bits, *source_bits((UNIT*)(s_map->bits));

	for(int r = 0;r< h; r++)
	{
		offset = ((offset-base)%total)+base;
		d_map->GetAddress(&dest_bits, 0, r*scale_y);
		
		if(r%2)
		{
			(*odd_func)(source_bits, offset, dest_bits, w, d_map->bits_per_row);
			offset--;
		}
		else
		{
			(*even_func)(source_bits, offset, dest_bits, w, d_map->bits_per_row);
		}
		offset+=s_map->bits_per_row;
	}
}


