#include "LifeWindow.h"
#include "MessageConstants.h"

#include "Maps.h"
static const BRect			FRAME(0,0, 511, 511);

class Window:public BWindow
	{
		bool	on_work_space;
		bool	is_minimized;
		bool	is_visible;
		BMessenger target;
		LifeWindow* life_window;
		
		void	UpdateVisible();
	public:
		Window(BRect, const BMessenger&, LifeWindow*);
		~Window();
//		void	MessageReceived(BMessage *message);
		bool	QuitRequested();
		void	WorkspaceActivated(int32 workspace, bool active);
		void	WorkspacesChanged(uint32 old_spaces, uint32 new_spaces);
		void	Minimize(bool minimize);
		void	WindowActivated(bool active);
		void	FrameResized();
//		void	FrameMoved();
	};

class View: public BView
	{
		BBitmap*	bit_map;
		BLocker*	buffer_locker;

	public:
		View(BBitmap*, BLocker*);
		~View();
		void Draw(BRect r);
//		void KeyDown(char*, int32);
//		void MouseDown(BPoint);
//		void AttachedToWindow();
//		void DetachedFromWindow();
//		void FrameResized(float, float);
	};

LifeWindow::LifeWindow(const BMessenger& target)
	: view(NULL)
	, buffer_enable(true)
	, window_locker(new BHandler("LifeWindow locker"))
	, buffer_locker(new BLocker("life window buffer"))
{
	bit_map = new BBitmap(FRAME, B_CMAP8, false, true);
//	bit_map = new BBitmap(FRAME, B_GRAY1, false, true);
	if(!bit_map->IsValid())
	{
		buffer_enable = false;
		(new BAlert("ERROR", "Can not allocate memory for bitmap", "OK"))->Go();
		return;
	}

	view = new View(bit_map, buffer_locker);
	BRect wf = FRAME.OffsetByCopy(20,100); wf.right-=4;
	BWindow* window = new Window(wf, target, this);
	window->AddChild(view);
	window->AddHandler(window_locker);
}


LifeWindow::~LifeWindow()
{
	if(window_locker->LockLooper())
	{//window is alive. destroy it.
		window_locker->Looper()->Quit();
	}
	delete window_locker;
	delete bit_map;
	delete buffer_locker;
}

bool
LifeWindow::InitCheck()
{
	return ((view!=NULL) && (bit_map->IsValid()) );
}

void
LifeWindow::Show()
{
	view->Window()->Show();
}
void
LifeWindow::Hide()
{
//	view->window->Hide();
}

void
LifeWindow::SyncScreen()
{
	if(window_locker->LockLooper())
	{
		view->Draw(view->Bounds());
		window_locker->UnlockLooper();
	}
}


void
LifeWindow::EnableBuffer(bool enable)
{
	BAutolock lock(buffer_locker);
	if(lock.IsLocked())
	{
		buffer_enable = enable;
	}
}

bool
LifeWindow::LockBuffer()
{
	BAutolock lock(buffer_locker);
	if(lock.IsLocked() && buffer_enable)
		return buffer_locker->Lock();
	return false;
}

void
LifeWindow::UnlockBuffer()
	{buffer_locker->Unlock();}



PixelMap
LifeWindow::GetBufferInfo()
{
	return GetPixelMap(bit_map);
}





/********************************************************************************/



Window::Window(BRect r, const BMessenger& target, LifeWindow* life_window)
	: BWindow(
		r,
		"life display",
		B_TITLED_WINDOW_LOOK,
		B_NORMAL_WINDOW_FEEL,
		0)
	, target(target)
	, life_window(life_window)
	, is_minimized(false)
	, on_work_space(true)
	, is_visible(true)
{
//	AddShortcut('F', B_COMMAND_KEY, new BMessage('Oooo'), this);
	SetSizeLimits(100, r.Width(), 100, r.Height());
	SetWindowAlignment(B_BYTE_ALIGNMENT, 4, 0, 4, 0);
}

Window::~Window()
{
}

bool Window::QuitRequested()
{
	life_window->EnableBuffer(false);
	target.SendMessage(MS_DISABLED);
	Minimize(true);
	return false;
}



void Window::UpdateVisible()
{
	bool v = (!is_minimized)&&on_work_space;
	if(is_visible!=v)
	{
		is_visible = v;
		life_window->EnableBuffer(v);
		if(v) target.SendMessage(MS_ENABLED);
		else	target.SendMessage(MS_DISABLED);
	}
}
void
Window::WorkspaceActivated(int32 workspace, bool active)
{//Invoked when current workspace is switched from/to the window laid. (false/true resp.)
	on_work_space = (active&&(workspace==current_workspace()));
	UpdateVisible();
}
void
Window::WorkspacesChanged(uint32 old_spaces, uint32 new_spaces)
{//Invoked when the workspace the window lays switched. (false/true resp.)
	on_work_space = (new_spaces >>current_workspace())&1;
	UpdateVisible();
}
void
Window::Minimize(bool minimize)
{//Invoked when the window is minimized. (Reverce is not true!...always minimize==true)
	BWindow::Minimize(minimize);is_minimized = minimize;
	UpdateVisible();
}
void
Window::WindowActivated(bool active)
{//Invoked when the window is active i.e. the titlebar highlighted and vice versa.
 // This is a repair for lack of hook function call Minimize(false).
	BWindow::WindowActivated(active); is_minimized = IsMinimized();
	UpdateVisible();
	target.SendMessage(MC_SET_VALUE);//Inform that BScreen is now available to the target.
}

void
Window::FrameResized()
{
	BMessage msg(MS_RESIZED);
	BRect r=Frame();
	msg.AddPoint("size", BPoint(r.Width(), r.Height()));
	target.SendMessage(&msg);
}

/*
void
Window::MessageReceived(BMessage *message)
{
	switch(message->what)
	{
	case 'Oooo':
		{
			target.SendMessage(MS_WINDOW_SCREEN);
		}
		break;
	default:
			BWindow::MessageReceived(message);
		break;
	}
}


*/
/********************************************************************************/


View::View(BBitmap* b, BLocker*l)
	: BView
		( b->Bounds().OffsetByCopy(-4,0)
		, "life view"
		, B_FOLLOW_NONE
		, B_WILL_DRAW
		)
	, bit_map(b)
	, buffer_locker(l)
{
	SetViewColor(B_TRANSPARENT_COLOR);
}

View::~View()
{}

void
View::Draw(BRect r)
{
	BAutolock l(buffer_locker);
	if(l.IsLocked()) DrawBitmap(bit_map, r, r);
}
/*
void
View::KeyDown(char* k, int32 l)
{
	if(l!=1) return;
	switch(l

}
*/

//#define GRID(x) (((x)+GRID_SIZE-1)/GRID_SIZE*GRID_SIZE)
//	int w = r.Width(), h = r.Height();
//	w = max_c(GRID(w), GRID_SIZE); h=max_c(GRID(h), GRID_SIZE);
//	bit_map = new BBitmap(BRect(0,0,w-1,h-1), B_CMAP8, false, true);
//void
//View::AttachedToWindow(){}
//
//void
//View::DetachedFromWindow(){}
//
//void
//View::FrameResized(float x, float y){}
/********************************************************************************/
